(herald "IKEv1 Pre-Shared-Key" (algebra diffie-hellman) (bound 15))

(comment "CPSA 3.4.0")
(comment "All input read from IKEv1_psk_a.scm")
(comment "Strand count bounded at 15")

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ni nr ci cr lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr Gr))
  (neq ((exp (gen) Gr) (gen)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni ci)
  (goals
    (forall
      ((iend node) (I R name) (ni nr ci cr lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (non (ltk R I)) (non (ltk I R)))
        (or
          (exists ((r1 node)) (and (p "resp" 0 r1) (p "resp" "R" r1 R)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 R))))))
    (forall
      ((iend node) (I R name) (ni nr ci cr lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (non (ltk R I)) (non (ltk I R)))
        (or
          (exists ((r1 node))
            (and (p "resp" 0 r1) (p "resp" "R" r1 R)
              (p "resp" "I" r1 I)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 R)
              (p "init" "R" i1 I))))))
    (forall
      ((iend node) (I R name) (ni nr ci cr lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (non (ltk R I)) (non (ltk I R)))
        (exists ((r1 node) (Gi base) (r expn))
          (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
            (p "resp" "Gi" r1 Gi) (p "resp" "r" r1 r)
            (= (exp Gi r) (exp Gr i))))))
    (forall
      ((iend node) (I R name) (ni nr ci cr lst algo text) (i expn)
        (Gr base))
      (implies
        (and (p "init" 2 iend) (p "init" "R" iend R)
          (p "init" "I" iend I) (p "init" "ni" iend ni)
          (p "init" "nr" iend nr) (p "init" "ci" iend ci)
          (p "init" "cr" iend cr) (p "init" "lst" iend lst)
          (p "init" "i" iend i) (p "init" "Gr" iend Gr)
          (p "init" "algo" iend algo) (non (ltk R I)) (non (ltk I R)))
        (exists ((r1 node) (r expn))
          (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
            (p "resp" "ni" r1 ni) (p "resp" "nr" r1 nr)
            (p "resp" "r" r1 r) (p "resp" "ci" r1 ci)
            (p "resp" "cr" r1 cr) (p "resp" "lst" r1 lst)
            (p "resp" "Gi" r1 (exp (gen) i))
            (p "resp" "algo" r1 algo))))))
  (traces
    ((send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ni nr ci cr lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi Gi))
  (absent (r (exp (gen) Gi)))
  (neq ((exp (gen) Gi) (gen)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr cr)
  (goals
    (forall
      ((rend node) (I R name) (ni nr ci cr lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (non (ltk R I)) (non (ltk I R)))
        (or
          (exists ((r1 node)) (and (p "resp" 0 r1) (p "resp" "R" r1 I)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 I))))))
    (forall
      ((rend node) (I R name) (ni nr ci cr lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (non (ltk R I)) (non (ltk I R)))
        (or
          (exists ((r1 node))
            (and (p "resp" 0 r1) (p "resp" "R" r1 I)
              (p "resp" "I" r1 R)))
          (exists ((i1 node))
            (and (p "init" 0 i1) (p "init" "I" i1 I)
              (p "init" "R" i1 R))))))
    (forall
      ((rend node) (I R name) (ni nr ci cr lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (non (ltk R I)) (non (ltk I R)))
        (exists ((i1 node) (Gr base) (i expn))
          (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
            (p "init" "Gr" i1 Gr) (p "init" "i" i1 i)
            (= (exp Gi r) (exp Gr i))))))
    (forall
      ((rend node) (I R name) (ni nr ci cr lst algo text) (r expn)
        (Gi base))
      (implies
        (and (p "resp" 3 rend) (p "resp" "R" rend R)
          (p "resp" "I" rend I) (p "resp" "ni" rend ni)
          (p "resp" "nr" rend nr) (p "resp" "ci" rend ci)
          (p "resp" "cr" rend cr) (p "resp" "lst" rend lst)
          (p "resp" "algo" rend algo) (p "resp" "r" rend r)
          (p "resp" "Gi" rend Gi) (non (ltk R I)) (non (ltk I R)))
        (exists ((i1 node) (i expn))
          (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
            (p "init" "ni" i1 ni) (p "init" "nr" i1 nr)
            (p "init" "ci" i1 ci) (p "init" "cr" i1 cr)
            (p "init" "algo" i1 algo) (p "init" "lst" i1 lst)
            (p "init" "Gr" i1 (exp (gen) r)) (p "init" "i" i1 i))))))
  (comment
    "Responder point of view; agreement / weak agreement / aliveness goals")
  (traces
    ((recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ni nr ci cr lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr Gr))
  (deflistener
    (hash "prf" (ltk I R) ni nr (exp (gen) (mul i Gr)) ci cr))
  (neq ((exp (gen) Gr) (gen)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni ci)
  (traces
    ((send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    ((recv (hash "prf" (ltk I R) ni nr (exp (gen) (mul i Gr)) ci cr))
      (send
        (hash "prf" (ltk I R) ni nr (exp (gen) (mul i Gr)) ci cr)))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ni nr ci cr lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand resp 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi Gi))
  (deflistener
    (hash "prf" (ltk I R) ni nr (exp (gen) (mul r Gi)) ci cr))
  (absent (r (exp (gen) Gi)))
  (neq ((exp (gen) Gi) (gen)))
  (non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr cr)
  (traces
    ((recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))))
    ((recv (hash "prf" (ltk I R) ni nr (exp (gen) (mul r Gi)) ci cr))
      (send
        (hash "prf" (ltk I R) ni nr (exp (gen) (mul r Gi)) ci cr)))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ci lst algo ni nr cr text) (I R name) (r i expn))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi i))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr r))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener (hash "prf" (ltk I R) ni nr (exp (gen) (mul r i)) ci cr))
  (precedes ((0 1) (1 1)) ((0 3) (2 0)) ((0 3) (3 0)) ((1 0) (0 0))
    ((1 2) (0 2)))
  (absent (r (exp (gen) i)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((exp (gen) r) (gen)) ((exp (gen) i) (gen)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen r i)
  (uniq-orig ci ni nr cr (ltk I R) (ltk R I))
  (traces
    ((recv (cat I R ci lst (exp (gen) i) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) i) cr ci lst R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) i) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) r) ci cr lst I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    ((send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) i) cr ci lst R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) r) ci cr lst I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) r) ci cr lst
            I)))) ((recv "blab") (send (ltk I R)))
    ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk I R) ni nr (exp (gen) (mul r i)) ci cr))
      (send (hash "prf" (ltk I R) ni nr (exp (gen) (mul r i)) ci cr)))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ci lst algo ni nr cr text) (I R name) (r i expn))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi i))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr r))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener (hash "prf" (ltk R I) ni nr (exp (gen) (mul r i)) ci cr))
  (precedes ((0 1) (1 1)) ((0 3) (2 0)) ((0 3) (3 0)) ((1 0) (0 0))
    ((1 2) (0 2)))
  (absent (r (exp (gen) i)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((exp (gen) r) (gen)) ((exp (gen) i) (gen)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen r i)
  (uniq-orig ci ni nr cr (ltk I R) (ltk R I))
  (traces
    ((recv (cat I R ci lst (exp (gen) i) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) i) cr ci lst R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) i) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) r) ci cr lst I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    ((send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) i) cr ci lst R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) r) ci cr lst I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) r) ci cr lst
            I)))) ((recv "blab") (send (ltk I R)))
    ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk R I) ni nr (exp (gen) (mul r i)) ci cr))
      (send (hash "prf" (ltk R I) ni nr (exp (gen) (mul r i)) ci cr)))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ni nr ci cr lst algo text) (I R name) (i expn) (Gr expr))
  (defstrand init 3 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (i i) (Gr Gr))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk I R) ni nr (exp (gen) (mul i Gr)) ci cr))
  (precedes ((0 2) (1 0)) ((0 2) (2 0)))
  (neq ((exp (gen) Gr) (gen)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i)
  (uniq-orig ni ci (ltk I R) (ltk R I))
  (traces
    ((send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)))) ((recv "blab") (send (ltk I R)))
    ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk I R) ni nr (exp (gen) (mul i Gr)) ci cr))
      (send
        (hash "prf" (ltk I R) ni nr (exp (gen) (mul i Gr)) ci cr)))))

(defprotocol pskAggr diffie-hellman
  (defrole init
    (vars (ni nr ci cr lst algo text) (i expn) (Gr expr) (I R name))
    (trace (send (cat I R ci lst (exp (gen) i) ni I))
      (recv
        (cat R I ci cr algo (exp (gen) Gr) nr R
          (hash (ltk I R) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) Gr) (exp (gen) i) cr ci lst
            R)))
      (send
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) i) (exp (gen) Gr) ci cr lst
            I))))
    (uniq-orig ni ci)
    (uniq-gen i)
    (neq ((exp (gen) Gr) (gen))))
  (defrole resp
    (vars (ni nr ci cr lst algo text) (r expn) (Gi expr) (I R name))
    (trace (recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab"))
    (uniq-orig nr cr)
    (uniq-gen r)
    (neq ((exp (gen) Gi) (gen)))
    (absent (r (exp (gen) Gi))))
  (defrole blabber-skey
    (vars (key skey))
    (trace (recv "blab") (send key))
    (uniq-orig key)))

(defskeleton pskAggr
  (vars (ni nr ci cr lst algo text) (I R name) (r expn) (Gi expr))
  (defstrand resp 4 (ni ni) (nr nr) (ci ci) (cr cr) (lst lst)
    (algo algo) (I I) (R R) (r r) (Gi Gi))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener
    (hash "prf" (ltk I R) ni nr (exp (gen) (mul r Gi)) ci cr))
  (precedes ((0 3) (1 0)) ((0 3) (2 0)))
  (absent (r (exp (gen) Gi)))
  (neq ((exp (gen) Gi) (gen)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen r)
  (uniq-orig nr cr (ltk I R) (ltk R I))
  (traces
    ((recv (cat I R ci lst (exp (gen) Gi) ni I))
      (send
        (cat R I ci cr algo (exp (gen) r) nr R
          (hash (ltk I R) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)
          (hash (ltk R I) ni nr (exp (gen) r) (exp (gen) Gi) cr ci lst
            R)))
      (recv
        (cat I R ci cr
          (hash (ltk I R) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I)
          (hash (ltk R I) ni nr (exp (gen) Gi) (exp (gen) r) ci cr lst
            I))) (send "blab")) ((recv "blab") (send (ltk I R)))
    ((recv "blab") (send (ltk R I)))
    ((recv (hash "prf" (ltk I R) ni nr (exp (gen) (mul r Gi)) ci cr))
      (send
        (hash "prf" (ltk I R) ni nr (exp (gen) (mul r Gi)) ci cr)))))
