;created as an attempt to replicate results found by Cremers as expressed in Key Exchange in IPsec Revisited.
;(Or rather from input of ikev2-child.cpp)
(herald "IKEv2 Child" (algebra diffie-hellman) (bound 8) (goals-sat) (limit 500))

(defmacro (SK_e) (hash I R secret "e"))
(defmacro (SK_ee) (hash R I secret "e"))
(defmacro (SK_d) (hash I R secret "d"))
(defmacro (SK_dd) (hash R I secret "d"))

;Macros to reflect what certian parties do not know
(defmacro (zi) (exp Gr i))
(defmacro (zr) (exp Gi r))

(defmacro (ilength) 2)
(defmacro (rlength) 1)

(defmacro (SKi) (hash "KDF" (SK_d)  ni nr))
(defmacro (SKii) (hash "KDF" (SK_dd)  ni nr))

(defprotocol child diffie-hellman
	     (defrole init
		      (vars (ni nr lst3 algo3 secret text) (i expn) (Gr base) (I R name))
		      (trace
			(send (cat I R (enc lst3 ni (SK_e)) (enc lst3 ni (SK_ee))))
			(recv (cat R I (enc algo3 nr (SK_e)) (enc algo3 nr (SK_ee))))
			(send "blab"))
		      (uniq-orig ni)
		      (neq (lst3 algo3)))
                      ;(priority (1 4)))
	     (defrole resp
		      (vars (ni nr lst3 algo3 secret text) (r expn) (Gi base) (I R name))
		      (trace
			(recv (cat I R (enc lst3 ni (SK_e)) (enc lst3 ni (SK_ee))))
			(send (cat R I (enc algo3 nr (SK_e)) (enc algo3 nr (SK_ee)))))
		      (uniq-orig nr))
                      ;(priority (0 4)))
	     (defrole blab-skey
               (vars (secret text))
               (trace
                (recv "blab")
                (send secret))
               (uniq-orig secret)))

;These were initially commented out because the strand bound was exceeded and making that larger made the file too big for my web browser to read well. 
(comment (defskeleton child
	     (vars (ni nr lst3 algo3 secret text) (i expn) (Gr base) (I R name))
	     (defstrand init 2 (I I) (R R) (secret secret))
	     (non-orig secret))

(defskeleton child
	     (vars (ni nr lst3 algo3 secret text) (r expn) (Gi base) (I R name))
	     (defstrand resp 2 (I I) (R R) (secret secret))
	     (non-orig secret)))





;Weakest - no keys are leaked. I have commented these out because this is not designed to have perfect forward secrecy
(defskeleton child
	     (vars (ni nr lst3 algo3 secret text) (I R name))
	     (defstrand init 2 (I I) (R R) (secret secret))
             (deflistener (SK_d))
	     (non-orig secret))

(defskeleton child
	     (vars (ni nr lst3 algo3 secret text) (I R name))
	     (defstrand init 2 (I I) (R R) (secret secret))
             (deflistener (SK_dd))
	     (non-orig secret))

(defskeleton child
	     (vars (ni nr lst3 algo3 secret text) (I R name))
	     (defstrand resp 2 (I I) (R R) (secret secret))
             (deflistener (SK_d))
	     (non-orig secret))

(defskeleton child
	     (vars (ni nr lst3 algo3 secret text) (I R name))
	     (defstrand resp 2 (I I) (R R) (secret secret))
             (deflistener (SK_dd))
	     (non-orig secret))

; Weak Forward Secrecy
(comment (defskeleton child
  (vars (ni nr lst3 algo3 text) (I R name) (r i expn) (secret text))
  (defstrand init 3 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gr (exp (gen) r)) (i i) (secret secret))
  (defstrand resp 2 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gi (exp (gen) i)) (r r) (secret secret))
  (defstrand blab-skey 2 (secret secret))
  (deflistener (SKf))
  (precedes ((0 0) (1 0)) ((1 1) (0 1)) ((0 2) (2 0)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((exp (gen) i) (gen)) ((exp (gen) r) (gen)))
  (uniq-gen r i)
  (uniq-orig ni nr))

(defskeleton child
  (vars (ni nr lst3 algo3 text) (I R name) (r i expn) (secret text))
  (defstrand init 3 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gr (exp (gen) r)) (i i) (secret secret))
  (defstrand resp 2 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gi (exp (gen) i)) (r r) (secret secret))
  (defstrand blab-skey 2 (secret secret))
  (deflistener (SKff))
  (precedes ((0 0) (1 0)) ((1 1) (0 1)) ((0 2) (2 0)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((exp (gen) i) (gen)) ((exp (gen) r) (gen)))
  (uniq-gen r i)
  (uniq-orig ni nr))

;Perfect forward secrecy
(defskeleton child
  (vars (ni nr lst3 algo3 text) (I R name) (i expn) (Gr base) (secret text))
  (defstrand init 3 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gr Gr) (i i) (secret secret))
     (defstrand blab-skey 2 (secret secret))
     (deflistener (SKi))
     (precedes ((0 2) (1 0))))

(defskeleton child
  (vars (ni nr lst3 algo3 text) (I R name) (i expn) (Gr base) (secret text))
  (defstrand init 3 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gr Gr) (i i) (secret secret))
     (defstrand blab-skey 2 (secret secret))
     (deflistener (SKii))
     (precedes ((0 2) (1 0))))

(defskeleton child
  (vars (ni nr lst3 algo3 secret text) (I R name) (r expn) (Gi base))
  (defstrand resp 2 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gi Gi) (r r) (secret secret))
     (defstrand blab-skey 2 (secret secret))
     (deflistener (SKr))
     (precedes ((0 1) (1 0))))

(defskeleton child
  (vars (ni nr lst3 algo3 secret text) (I R name) (r expn) (Gi base))
  (defstrand resp 2 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gi Gi) (r r) (secret secret))
     (defstrand blab-skey 2 (secret secret))
     (deflistener (SKrr))
     (precedes ((0 1) (1 0))))


;Perfect forward secrecy
(defskeleton child
  (vars (ni nr lst3 algo3 text) (I R name) (r i expn))
  (defstrand init 3 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gr (exp (gen) r)) (i i))
     (defstrand blab-skey 2 (key (ltk R I)))
     (defstrand blab-skey 2 (key (ltk I R)))
     (precedes ((0 2) (1 0)) ((0 2) (2 0)))
     (pen-non-orig (ltk I R) (ltk R I)))

(defskeleton child
  (vars (ni nr lst3 algo3 text) (I R name) (r i expn))
  (defstrand resp 2 (ni ni) (nr nr) (lst3 lst3) (algo3 algo3) (I I)
    (R R) (Gi (exp (gen) i)) (r r))
     (defstrand blab-skey 2 (key (ltk R I)))
     (defstrand blab-skey 2 (key (ltk I R)))
     (precedes ((0 1) (1 0)) ((0 1) (2 0)))
     (pen-non-orig (ltk I R) (ltk R I))))

(defgoal child
  ; Aliveness - my peer has taken an action
  (forall ((iend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "secret" iend secret)
		(non secret))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 R)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 R))))))
  ; Weak agreement - my peer and I agree on our names
  ; Aliveness - my peer has taken an action
  (forall ((iend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "secret" iend secret)
		(non secret))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 R) (p "init" "R" i1 I))))))
  ; Minimal agreement - my peer and I agree on our names and roles, and on the DH key
  ; Aliveness - my peer has taken an action
  (forall ((iend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "secret" iend secret)
		(non secret))
	   (exists ((r1 node))
		   (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
			))))

  )(comment
  ; Full agreement - my peer and I agree on the full transcript
  ; Aliveness - my peer has taken an action
  (forall ((iend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "secret" iend secret)
		(non secret))
	   (exists ((r1 node))
		   (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
			(p "resp" "ni" r1 ni) (p "resp" "nr" r1 nr)
			(p "resp" "lst3" r1 lst3) (p "resp" "algo3" r1 algo3)
			(p "resp" "secret" r1 secret)))))
  )

(defgoal child
  ; Aliveness - my peer has taken an action
  (forall ((rend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "secret" rend secret)
		(p "resp" "lst3" rend lst3) (p "resp" "algo3" rend algo3)
		(non secret))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 I)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 I))))))
  ; Weak agreement - my peer and I agree on our names
  (forall ((rend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "secret" rend secret)
		(p "resp" "lst3" rend lst3) (p "resp" "algo3" rend algo3)
		(non secret))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 I) (p "resp" "I" r1 R)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 I) (p "init" "R" i1 R))))))
  ; Minimal agreement - my peer and I agree on our names and roles, and on the DH key
  (forall ((rend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "secret" rend secret)
		(p "resp" "lst3" rend lst3) (p "resp" "algo3" rend algo3)
		(non secret))
	   (exists ((i1 node))
		   (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
			))))
  ; Full agreement - my peer and I agree on the full transcript
  (forall ((rend node) (I R name) (ni nr lst3 algo3 secret text))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "secret" rend secret)
		(p "resp" "lst3" rend lst3) (p "resp" "algo3" rend algo3)
		(non secret))
	   (exists ((i1 node))
		   (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
			(p "init" "ni" i1 ni) (p "init" "nr" i1 nr)
			(p "init" "algo3" i1 algo3) (p "init" "lst3" i1 lst3)
			(p "init" "secret" i1 secret) 
			))))
  (comment "Responder point of view; agreement / weak agreement / aliveness goals")
  )

