(herald "IADH: unified model (UM)" (bound 20) (limit 2000)
  (algebra diffie-hellman))

(comment "CPSA 3.5.0")
(comment "All input read from iadh_um.scm")
(comment "Strand count bounded at 20")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (lp ep expt) (self peer name) (key data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ep)) (send key))
    (uniq-gen e)
    (fn-of
      (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e ep))) key)))
    (neq (ep (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) lp))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA lA eB lB rndx)
    (lp ep lp-0 ep-0 expt))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp lp) (ep ep))
  (defstrand participant 5 (key key) (self B) (peer D) (l lB) (e eB)
    (lp lp-0) (ep ep-0))
  (absent (eB (exp (gen) lB)) (eB (exp (gen) lp-0)) (eA (exp (gen) lA))
    (eA (exp (gen) lp)))
  (fn-of
    (foo
      ((hash (exp (gen) (mul lB lp-0)) (exp (gen) (mul eB ep-0))) key)
      ((hash (exp (gen) (mul lA lp)) (exp (gen) (mul eA ep))) key)))
  (neq (ep-0 (one)) (eA eB) (ep (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) ep)) (send key))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp (gen) lp-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep-0)) (send key)))
  (label 0)
  (unrealized (0 0) (0 1) (1 0) (1 1))
  (preskeleton)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA lB rndx) (lp expt) (eA eB rndx)
    (ep expt))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) lB lp)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l lB) (e eB)
    (lp lp) (ep ep))
  (precedes ((1 2) (0 3)))
  (absent (eB (exp (gen) lB)) (eB (exp (gen) lp)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) lB lp))))
  (fn-of
    (foo ((hash (exp (gen) (mul lB lp)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) lB lp)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp (gen) lp) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key)))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1) (0 3) (1 0) (1 1))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA rndx) (lp expt) (eA eB rndx)
    (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) lp l)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp lp) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) lp)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) lp l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul lp l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l) B (privk B)) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) lp l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) lp) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 2)
  (parent 1)
  (unrealized (0 0) (0 1) (0 3) (1 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA eB rndx) (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) l l)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 3)
  (parent 2)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB rndx) (ep expt)
    (l l-0 rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) l l-0)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 4)
  (parent 2)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA eB rndx) (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer C) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk C) l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 5)
  (parent 3)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l l-0) (e eA)
    (lp (mul l l (rec l-0))) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 6)
  (parent 3)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer C) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 7)
  (parent 4)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer C) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 8)
  (parent 4)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB rndx) (ep expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l l-1) (e eA)
    (lp (mul l l-0 (rec l-1))) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 9)
  (parent 4)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 10)
  (parent 5)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 11)
  (parent 7)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 12)
  (parent 8)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eB l e)
  (operation nonce-test (displaced 3 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 13)
  (parent 10)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eA))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq (eA (one)) (eA e) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l e)
  (operation nonce-test (displaced 3 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 14)
  (parent 10)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C B) (D B) (lA l) (lB l) (lp l) (eA eA)
        (eB e) (ep eA))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 2) (0 3)) ((3 2) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 15)
  (parent 10)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 3 2 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 16)
  (parent 10)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 17)
  (parent 10)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (ep expt) (l rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
    (exp (gen) (mul (rec eA) eB ep)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))))
  (label 18)
  (parent 10)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 19)
  (parent 11)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eA))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq (eA (one)) (eA e) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 20)
  (parent 11)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C D) (D D) (lA l) (lB l) (lp l-0) (eA eA)
        (eB e) (ep eA))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 21)
  (parent 11)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 22)
  (parent 11)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 23)
  (parent 11)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 24)
  (parent 11)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx)
    (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
    (exp (gen) (mul (rec eA) eB ep)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))))
  (label 25)
  (parent 11)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 26)
  (parent 12)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eA))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq (eA (one)) (eA e) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 27)
  (parent 12)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A D) (B B) (C B) (D D) (lA l-0) (lB l) (lp l-0)
        (eA eA) (eB e) (ep eA))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 28)
  (parent 12)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 29)
  (parent 12)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 30)
  (parent 12)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 31)
  (parent 12)
  (unrealized (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx)
    (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
    (exp (gen) (mul (rec eA) eB ep)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))))
  (label 32)
  (parent 12)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eB l e)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eB) e e (rec w))) w))))
  (label 33)
  (parent 13)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))))
  (label 34)
  (parent 15)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l (rec w))) w))
    (exp (gen) (mul eA (rec eB) l)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l (rec w))) w))))
  (label 35)
  (parent 16)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))))
  (label 36)
  (parent 17)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (ep (mul eA (rec eB) w)) (w-0 w))
    (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 37)
  (parent 18)
  (unrealized (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 38)
  (parent 18)
  (unrealized (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 39)
  (parent 18)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 40)
  (parent 18)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((mul eA (rec eB) w l) (one)) (eA eB) ((mul w l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 41)
  (parent 18)
  (unrealized (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 42)
  (parent 18)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB l l-0 e)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eB) e e (rec w))) w))))
  (label 43)
  (parent 19)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))))
  (label 44)
  (parent 21)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))))
  (label 45)
  (parent 22)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))))
  (label 46)
  (parent 23)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))))
  (label 47)
  (parent 24)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (ep (mul eA (rec eB) w)) (w-0 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 48)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 49)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 50)
  (parent 25)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 51)
  (parent 25)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 52)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 53)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 54)
  (parent 25)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB l l-0 e)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eB) e e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eB) e e (rec w))) w))))
  (label 55)
  (parent 26)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) e (rec w))) w))))
  (label 56)
  (parent 28)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))))
  (label 57)
  (parent 29)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-0 (rec w))) w))))
  (label 58)
  (parent 30)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))
      (send (cat (exp (gen) (mul eA (rec eB) l-1 (rec w))) w))))
  (label 59)
  (parent 31)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (ep (mul eA (rec eB) w)) (w-0 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 60)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 61)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 62)
  (parent 32)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 63)
  (parent 32)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 64)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 65)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 66)
  (parent 32)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eB e)
  (precur (3 0))
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 67)
  (parent 33)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 68)
  (parent 33)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-1)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 69)
  (parent 33)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 70)
  (parent 33)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l))))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l))))))
  (label 71)
  (parent 33)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 72)
  (parent 33)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 73)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eB e l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul (rec eB) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul (rec eB) e e-0) (one)) (e-0 eB) (e (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e)))))
  (label 74)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA e l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 75)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA eB l-0 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 5 3 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 76)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 77)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))))
  (label 78)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 79)
  (parent 34)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-0 l) (w (mul eA (rec eB) l)))
    (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) l)))
      (send (cat (gen) (mul eA (rec eB) l)))))
  (label 80)
  (parent 35)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) l e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l e))) key)))
  (neq ((mul (rec eB) l e) (one)) (e eB) (l (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 81)
  (parent 35)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 82)
  (parent 35)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 83)
  (parent 35)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB))))
      (send (cat (exp (gen) l) (mul eA (rec eB))))))
  (label 84)
  (parent 35)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 85)
  (parent 35)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 86)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul (rec eB) l e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l e))) key)))
  (neq ((mul (rec eB) l e) (one)) (e eB) (l (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 87)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 88)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 89)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 90)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 91)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 92)
  (parent 36)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) eB))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) eB)) (send (cat (gen) eB))))
  (label 93)
  (parent 37)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 94)
  (parent 37)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 95)
  (parent 37)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq (l (one)) (eA eB) ((mul (rec eA) eB l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB l)))
      (send (cat (gen) (mul (rec eA) eB l)))))
  (label 96)
  (parent 37)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 97)
  (parent 37)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx) (w w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))))
  (label 98)
  (parent 37)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e))))))
  (label 99)
  (parent 38)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (e e-0) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 100)
  (parent 38)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e eB) ((mul eB (rec e) e-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 101)
  (parent 38)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq (l (one)) (e eB) ((mul eB (rec e) l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l)))))
  (label 102)
  (parent 38)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 103)
  (parent 38)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eB) w e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))))
  (label 104)
  (parent 38)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l e)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 105)
  (parent 39)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C B) (D B) (lA l) (lB l) (lp l) (eA eA)
        (eB e) (ep (mul eA w)))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eB e l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e-0 eB) (eB (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 106)
  (parent 40)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA e l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (eA e-0) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 107)
  (parent 40)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA eB l-0 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 5 4 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 108)
  (parent 40)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0))
    ((5 2) (1 3)) ((5 2) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (eA eB) ((mul (rec eA) eB e-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 109)
  (parent 40)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 110)
  (parent 40)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0))
    ((5 0) (1 3)) ((5 0) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 111)
  (parent 40)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))))
  (label 112)
  (parent 40)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l))))))
  (label 113)
  (parent 41)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 114)
  (parent 41)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 115)
  (parent 41)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq (l (one)) (eA eB) ((mul (rec eA) eB l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB)))
      (send (cat (exp (gen) l) (mul (rec eA) eB)))))
  (label 116)
  (parent 41)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 117)
  (parent 41)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((mul eA (rec eB) w l) (one)) (eA eB) ((mul w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))))
  (label 118)
  (parent 41)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 119)
  (parent 42)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 120)
  (parent 42)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0))
    ((5 2) (1 3)) ((5 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 121)
  (parent 42)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 122)
  (parent 42)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 123)
  (parent 42)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0))
    ((5 0) (1 3)) ((5 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 124)
  (parent 42)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 125)
  (parent 42)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eB e)
  (precur (4 0))
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 126)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 127)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 128)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 129)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 130)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 131)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 132)
  (parent 43)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 133)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eB e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul (rec eB) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul (rec eB) e e-0) (one)) (e-0 eB) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e)))))
  (label 134)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 135)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB l-1 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 136)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 137)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 138)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 139)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 140)
  (parent 44)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 141)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) l e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l e))) key)))
  (neq ((mul (rec eB) l e) (one)) (e eB) (l (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 142)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 143)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 144)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 145)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 146)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 147)
  (parent 45)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 148)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) l-0 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 e))) key)))
  (neq ((mul (rec eB) l-0 e) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 149)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 150)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 151)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 152)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 153)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 154)
  (parent 46)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1)))))
  (label 155)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul (rec eB) l e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l e))) key)))
  (neq ((mul (rec eB) l e) (one)) (e eB) (l (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 156)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 157)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 158)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 159)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 160)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB))))))
  (label 161)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 162)
  (parent 47)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) eB))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) eB)) (send (cat (gen) eB))))
  (label 163)
  (parent 48)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 164)
  (parent 48)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 165)
  (parent 48)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 166)
  (parent 48)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 167)
  (parent 48)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 168)
  (parent 48)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))))
  (label 169)
  (parent 48)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e))))))
  (label 170)
  (parent 49)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (e e-0) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 171)
  (parent 49)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e eB) ((mul eB (rec e) e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 172)
  (parent 49)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 173)
  (parent 49)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 174)
  (parent 49)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (e eB) ((mul eB (rec e) l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 175)
  (parent 49)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eB) w e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))))
  (label 176)
  (parent 49)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA l l-0 e)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 177)
  (parent 50)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C D) (D D) (lA l) (lB l) (lp l-0) (eA eA)
        (eB e) (ep (mul eA w)))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eB e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e-0 eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 178)
  (parent 51)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (eA e-0) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 179)
  (parent 51)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB l-1 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 180)
  (parent 51)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (eA eB) ((mul (rec eA) eB e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 181)
  (parent 51)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 182)
  (parent 51)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 183)
  (parent 51)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 184)
  (parent 51)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))))
  (label 185)
  (parent 51)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l))))))
  (label 186)
  (parent 52)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 187)
  (parent 52)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 188)
  (parent 52)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 189)
  (parent 52)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 190)
  (parent 52)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 191)
  (parent 52)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 192)
  (parent 52)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0))))))
  (label 193)
  (parent 53)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 194)
  (parent 53)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 195)
  (parent 53)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 196)
  (parent 53)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 197)
  (parent 53)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 198)
  (parent 53)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 199)
  (parent 53)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 200)
  (parent 54)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 201)
  (parent 54)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 202)
  (parent 54)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 203)
  (parent 54)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 204)
  (parent 54)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 205)
  (parent 54)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 206)
  (parent 54)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))))
  (label 207)
  (parent 54)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eB e)
  (precur (4 0))
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 208)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 209)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 210)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 211)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 212)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 213)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 214)
  (parent 55)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 215)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eB e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul (rec eB) e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul (rec eB) e e-0) (one)) (e-0 eB) (e (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e)))))
  (label 216)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 217)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB l-1 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 218)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 219)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 220)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 221)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 222)
  (parent 56)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 223)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) l-0 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 e))) key)))
  (neq ((mul (rec eB) l-0 e) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 224)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 225)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 226)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 227)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 228)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 229)
  (parent 57)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 230)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) l e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l e))) key)))
  (neq ((mul (rec eB) l e) (one)) (e eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 231)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 232)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 233)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 234)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 235)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 236)
  (parent 58)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1)))))
  (label 237)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul (rec eB) l e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l e))) key)))
  (neq ((mul (rec eB) l e) (one)) (e eB) (l (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) l e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 238)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 239)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 240)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 241)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 242)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB))))))
  (label 243)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 244)
  (parent 59)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) eB))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) eB)) (send (cat (gen) eB))))
  (label 245)
  (parent 60)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 246)
  (parent 60)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 247)
  (parent 60)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 248)
  (parent 60)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 249)
  (parent 60)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 250)
  (parent 60)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))))
  (label 251)
  (parent 60)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e))))))
  (label 252)
  (parent 61)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (e e-0) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 253)
  (parent 61)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e eB) ((mul eB (rec e) e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 254)
  (parent 61)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 255)
  (parent 61)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 256)
  (parent 61)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (e eB) ((mul eB (rec e) l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 257)
  (parent 61)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eB) w e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))))
  (label 258)
  (parent 61)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA l l-0 e)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 259)
  (parent 62)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A D) (B B) (C B) (D D) (lA l-0) (lB l) (lp l-0)
        (eA eA) (eB e) (ep (mul eA w)))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eB e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e-0 eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 260)
  (parent 63)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (eA e-0) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 261)
  (parent 63)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB l-1 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 262)
  (parent 63)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (eA eB) ((mul (rec eA) eB e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 263)
  (parent 63)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 264)
  (parent 63)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 265)
  (parent 63)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 266)
  (parent 63)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))))
  (label 267)
  (parent 63)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0))))))
  (label 268)
  (parent 64)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 269)
  (parent 64)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 270)
  (parent 64)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 271)
  (parent 64)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 272)
  (parent 64)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 273)
  (parent 64)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 274)
  (parent 64)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l))))))
  (label 275)
  (parent 65)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 276)
  (parent 65)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 277)
  (parent 65)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 278)
  (parent 65)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 279)
  (parent 65)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 280)
  (parent 65)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 281)
  (parent 65)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 282)
  (parent 66)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 283)
  (parent 66)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 284)
  (parent 66)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 285)
  (parent 66)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 286)
  (parent 66)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 287)
  (parent 66)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 288)
  (parent 66)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))))
  (label 289)
  (parent 66)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB w)) (w-1 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 290)
  (parent 98)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 291)
  (parent 98)
  (unrealized (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 292)
  (parent 98)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w e)))
      (send (cat (gen) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 293)
  (parent 98)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w l)))
      (send (cat (gen) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 294)
  (parent 98)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 295)
  (parent 98)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w-0 (mul eB (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 296)
  (parent 104)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w (rec e))))
      (send (cat (exp (gen) e) (mul eB w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 297)
  (parent 104)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((mul w e-0) (one)) (e e-0) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 298)
  (parent 104)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (e eB) ((mul eB (rec e) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 299)
  (parent 104)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (e eB) ((mul eB (rec e) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 300)
  (parent 104)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 301)
  (parent 104)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec e) w))
      (e-0 e) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 302)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e-0)))
        key)))
  (neq ((mul w e-0) (one)) (e-0 eB) ((mul eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 303)
  (parent 112)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((mul w e-0) (one)) (eA e-0) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 304)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA eB rndx) (w expt) (l-0 rndx)
    (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 305)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (eA eB) ((mul (rec eA) eB w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 306)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0)))
        key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 307)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 308)
  (parent 112)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l) w))
      (l-0 l) (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 309)
  (parent 118)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w (rec l))))
      (send (cat (exp (gen) l) (mul eB w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 310)
  (parent 118)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 311)
  (parent 118)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 312)
  (parent 118)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 313)
  (parent 118)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 314)
  (parent 118)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 315)
  (parent 125)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 316)
  (parent 125)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 317)
  (parent 125)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 318)
  (parent 125)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0)))
        key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 319)
  (parent 125)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 320)
  (parent 125)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 321)
  (parent 125)
  (unrealized (0 3) (3 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 322)
  (parent 169)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 323)
  (parent 169)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 324)
  (parent 169)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w e)))
      (send (cat (gen) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 325)
  (parent 169)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 326)
  (parent 169)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 327)
  (parent 169)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-1)))
      (send (cat (gen) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 328)
  (parent 169)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w-0 (mul eB (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 329)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w (rec e))))
      (send (cat (exp (gen) e) (mul eB w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 330)
  (parent 176)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((mul w e-0) (one)) (e e-0) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 331)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (e eB) ((mul eB (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 332)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 333)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 334)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (e eB) ((mul eB (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 335)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 336)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e-0)))
        key)))
  (neq ((mul w e-0) (one)) (e-0 eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 337)
  (parent 185)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((mul w e-0) (one)) (eA e-0) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 338)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 339)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (eA eB) ((mul (rec eA) eB w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 340)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 341)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 342)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 343)
  (parent 185)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 344)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB w (rec l))))
      (send (cat (exp (gen) l) (mul eB w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 345)
  (parent 192)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 346)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 347)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 348)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 349)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 350)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 351)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 352)
  (parent 199)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 353)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 354)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 355)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 356)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 357)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 358)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 359)
  (parent 207)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 360)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 361)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 362)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 363)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 364)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 365)
  (parent 207)
  (unrealized (0 3) (4 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 366)
  (parent 251)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 367)
  (parent 251)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 368)
  (parent 251)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w e)))
      (send (cat (gen) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 369)
  (parent 251)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 370)
  (parent 251)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 371)
  (parent 251)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-1)))
      (send (cat (gen) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 372)
  (parent 251)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w-0 (mul eB (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 373)
  (parent 258)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w (rec e))))
      (send (cat (exp (gen) e) (mul eB w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 374)
  (parent 258)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((mul w e-0) (one)) (e e-0) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 375)
  (parent 258)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (e eB) ((mul eB (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 376)
  (parent 258)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 377)
  (parent 258)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 378)
  (parent 258)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (e eB) ((mul eB (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 379)
  (parent 258)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 380)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e-0)))
        key)))
  (neq ((mul w e-0) (one)) (e-0 eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 381)
  (parent 267)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((mul w e-0) (one)) (eA e-0) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 382)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 383)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (eA eB) ((mul (rec eA) eB w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 384)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 385)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 386)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 387)
  (parent 267)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 388)
  (parent 274)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 389)
  (parent 274)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 390)
  (parent 274)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 391)
  (parent 274)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 392)
  (parent 274)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 393)
  (parent 274)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 394)
  (parent 274)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 395)
  (parent 281)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB w (rec l))))
      (send (cat (exp (gen) l) (mul eB w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 396)
  (parent 281)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 397)
  (parent 281)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 398)
  (parent 281)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 399)
  (parent 281)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 400)
  (parent 281)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 401)
  (parent 281)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 402)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 403)
  (parent 289)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 404)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 405)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 406)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 407)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 408)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 409)
  (parent 289)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 410)
  (parent 290)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eB eA)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 411)
  (parent 290)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 412)
  (parent 290)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 413)
  (parent 291)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 414)
  (parent 291)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener eB) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eB) (send eB)))
  (label 415)
  (parent 291)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec e) l-0) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 416)
  (parent 292)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 417)
  (parent 292)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 418)
  (parent 292)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 419)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 420)
  (parent 293)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 421)
  (parent 293)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 422)
  (parent 293)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 423)
  (parent 294)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l))) (send (cat (gen) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 424)
  (parent 294)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB))))))
  (label 425)
  (parent 294)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul (rec eA) eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-absence l (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 426)
  (parent 294)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 427)
  (parent 295)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 428)
  (parent 295)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 429)
  (parent 295)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 430)
  (parent 295)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 431)
  (parent 295)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 432)
  (parent 296)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (gen) (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (gen) (mul (rec eB) e e l-0)))
      (send (cat (gen) (mul (rec eB) e e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 433)
  (parent 296)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 434)
  (parent 296)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 435)
  (parent 296)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) e)) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 436)
  (parent 297)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 437)
  (parent 297)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul w (rec e)))
    (mul w (rec e)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 438)
  (parent 297)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 439)
  (parent 297)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 440)
  (parent 298)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e e-0 l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul e e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e e (rec e-0) l-0) (one)) (e e-0) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 441)
  (parent 298)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 442)
  (parent 298)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 443)
  (parent 298)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul (rec eB) e e (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 444)
  (parent 299)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 445)
  (parent 299)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (e rndx)
    (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 446)
  (parent 299)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 447)
  (parent 299)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 448)
  (parent 299)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l)))
      (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l))))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l))))))
  (label 449)
  (parent 300)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB e l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 450)
  (parent 300)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (e eB) ((mul (rec e) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l))
    (mul (rec e) (rec e) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB))))))
  (label 451)
  (parent 300)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul eB w l)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l e e))) key)))
  (neq ((mul w l e e) (one)) (e eB) ((mul eB w l e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-absence e (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l)))
      (send (cat (exp (gen) e) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w e e)))
      (send (cat (exp (gen) l) (mul w e e)))))
  (label 452)
  (parent 300)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (added-absence l (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 453)
  (parent 300)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 454)
  (parent 301)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 455)
  (parent 301)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 456)
  (parent 301)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 457)
  (parent 301)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 458)
  (parent 301)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 459)
  (parent 301)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 460)
  (parent 302)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-1)))
      (send (cat (gen) (mul eA (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 461)
  (parent 302)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 462)
  (parent 302)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA e rndx)
    (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 463)
  (parent 302)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 464)
  (parent 302)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (l-0 e eB e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep e-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul e (rec eB) e-0) (one)) (e eB) (e-0 (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e-0) (w (mul (rec eB) e-0))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0)))))
  (label 465)
  (parent 303)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 e eB e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul e-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0 l-1)))
        key)))
  (neq ((mul e (rec eB) e-0 l-1) (one)) (e eB) ((mul e-0 l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eB e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 466)
  (parent 303)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w))
    (mul (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB))))))
  (label 467)
  (parent 303)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 468)
  (parent 303)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (l-0 eA e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 469)
  (parent 304)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 eA e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eA e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA e (rec e-0) l-1) (one)) (eA e-0) ((mul e l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 470)
  (parent 304)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 471)
  (parent 304)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 472)
  (parent 304)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 473)
  (parent 304)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (e eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e eA))) key)))
  (neq ((mul e eA (rec eB)) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 474)
  (parent 305)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (e eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e eA l-1))) key)))
  (neq ((mul e eA (rec eB) l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 475)
  (parent 305)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eB rndx) (w expt) (l-0 rndx)
    (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 476)
  (parent 305)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA rndx) (w expt) (l-0 rndx)
    (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 477)
  (parent 305)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul eA (rec eB) e (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 478)
  (parent 306)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 479)
  (parent 306)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0 eA))) key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w e-0))
    (mul eB (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 480)
  (parent 306)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 481)
  (parent 306)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0))) key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 482)
  (parent 306)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 483)
  (parent 306)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-0))) (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))))
  (label 484)
  (parent 307)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 485)
  (parent 307)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-0))
    (mul eB (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 486)
  (parent 307)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-0))
    (mul (rec eA) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 487)
  (parent 307)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB rndx)
    (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 e)))
        key)))
  (neq ((mul w l-0 e) (one)) (eA eB) ((mul (rec eA) eB w l-0 e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w e)))
      (send (cat (exp (gen) l-0) (mul w e)))))
  (label 488)
  (parent 307)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 489)
  (parent 307)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 490)
  (parent 308)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 491)
  (parent 308)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 492)
  (parent 308)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 493)
  (parent 308)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 494)
  (parent 308)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 e))) key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 495)
  (parent 308)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 496)
  (parent 308)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-0 l) (w (mul eA (rec eB) l)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l)))
      (send (cat (gen) (mul eA (rec eB) l)))))
  (label 497)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l l-0)))
      (send (cat (gen) (mul eA (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 498)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB (rec l) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w))
    (mul eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 499)
  (parent 309)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 500)
  (parent 309)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul (rec eA) eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-absence l (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l))) (send (cat (gen) (mul w l)))))
  (label 501)
  (parent 309)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l))) key)))
  (neq ((mul e (rec eB) l) (one)) (e eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e eB l)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l)) (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 502)
  (parent 310)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (e eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0))) key)))
  (neq ((mul e (rec eB) l l-0) (one)) (e eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 503)
  (parent 310)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul w (rec l)))
    (mul w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 504)
  (parent 310)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l))) key)))
  (neq ((mul w e l) (one)) (e eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (added-absence l (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w)))
      (send (cat (exp (gen) l) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 505)
  (parent 310)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l (rec e) (rec e))) (l-0 l)
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 506)
  (parent 311)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA l (rec e) l-0) (one)) (eA e) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 507)
  (parent 311)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 508)
  (parent 311)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul (rec eA) w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l))) key)))
  (neq ((mul w e l) (one)) (eA e) ((mul (rec eA) w e e l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (added-absence l (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 509)
  (parent 311)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 510)
  (parent 311)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0)
      (w (mul eA (rec eB) l-0 (rec e))) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 511)
  (parent 312)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 512)
  (parent 312)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB l-0 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul eB (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eB l-0 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w e))
    (mul eB (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 513)
  (parent 312)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA l-0 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w e))
    (mul (rec eA) (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 514)
  (parent 312)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB rndx)
    (w expt) (e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (l-0 (mul (rec eA) eB w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (eA eB) ((mul (rec eA) eB w e l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 515)
  (parent 312)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (mul (rec eA) eB (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 516)
  (parent 312)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l eA))) key)))
  (neq ((mul l eA (rec eB)) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB))))
      (send (cat (exp (gen) l) (mul eA (rec eB))))))
  (label 517)
  (parent 313)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l eA l-0))) key)))
  (neq ((mul l eA (rec eB) l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 518)
  (parent 313)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w)))
      (send (cat (exp (gen) l) (mul eB w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 519)
  (parent 313)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB))))))
  (label 520)
  (parent 313)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 521)
  (parent 314)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 522)
  (parent 314)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 523)
  (parent 314)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 524)
  (parent 314)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 525)
  (parent 314)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 526)
  (parent 314)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 527)
  (parent 314)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 528)
  (parent 315)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 529)
  (parent 315)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 530)
  (parent 315)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 531)
  (parent 315)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 532)
  (parent 315)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 533)
  (parent 315)
  (unrealized (0 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (l-1 l-0) (w (mul (rec eB) l-0))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 534)
  (parent 316)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0))) key)))
  (neq ((mul e (rec eB) l-0 l-0) (one)) (e eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 535)
  (parent 316)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 536)
  (parent 316)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w))
    (mul (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 537)
  (parent 316)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 538)
  (parent 316)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul eA l-0 (rec e) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 539)
  (parent 317)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) (eA e) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 540)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 541)
  (parent 317)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 542)
  (parent 317)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 543)
  (parent 317)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 544)
  (parent 317)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 545)
  (parent 318)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1))) key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 546)
  (parent 318)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer self-1 name) (l l-0 rndx)
    (lp expt) (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2))) key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 547)
  (parent 318)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 548)
  (parent 318)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 549)
  (parent 318)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 550)
  (parent 318)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 551)
  (parent 318)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 552)
  (parent 319)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 553)
  (parent 319)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l l-1)))
        key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 554)
  (parent 319)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 555)
  (parent 319)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 556)
  (parent 319)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0))) key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 557)
  (parent 319)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 558)
  (parent 319)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 559)
  (parent 320)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 560)
  (parent 320)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 561)
  (parent 320)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 562)
  (parent 320)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 563)
  (parent 320)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 564)
  (parent 321)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1))) key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 565)
  (parent 321)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 566)
  (parent 321)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-2))) key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 567)
  (parent 321)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 568)
  (parent 321)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 569)
  (parent 321)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 570)
  (parent 321)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 571)
  (parent 321)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 572)
  (parent 322)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 573)
  (parent 322)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 574)
  (parent 322)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 575)
  (parent 322)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 576)
  (parent 323)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 577)
  (parent 323)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 578)
  (parent 323)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eB) (send eB)))
  (label 579)
  (parent 323)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec e) l-0) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 580)
  (parent 324)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec e) l-1) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 581)
  (parent 324)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 582)
  (parent 324)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 583)
  (parent 324)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 584)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 585)
  (parent 325)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 586)
  (parent 325)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 587)
  (parent 325)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 588)
  (parent 325)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 589)
  (parent 326)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 590)
  (parent 326)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 591)
  (parent 326)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 592)
  (parent 326)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 593)
  (parent 326)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 594)
  (parent 327)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 595)
  (parent 327)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 596)
  (parent 327)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 597)
  (parent 327)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 598)
  (parent 327)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 599)
  (parent 328)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 600)
  (parent 328)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 601)
  (parent 328)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 602)
  (parent 328)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 603)
  (parent 328)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 604)
  (parent 328)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 605)
  (parent 329)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (gen) (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (gen) (mul (rec eB) e e l-0)))
      (send (cat (gen) (mul (rec eB) e e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 606)
  (parent 329)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eB) e e l-1)))
      (send (cat (gen) (mul (rec eB) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 607)
  (parent 329)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 608)
  (parent 329)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 609)
  (parent 329)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 610)
  (parent 330)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 611)
  (parent 330)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 612)
  (parent 330)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 613)
  (parent 330)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 614)
  (parent 330)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 615)
  (parent 331)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e e-0 l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e e (rec e-0) l-0) (one)) (e e-0) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 616)
  (parent 331)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e e (rec e-0) l-1) (one)) (e e-0) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 617)
  (parent 331)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 618)
  (parent 331)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 619)
  (parent 331)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul (rec eB) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 620)
  (parent 332)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 621)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 622)
  (parent 332)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 623)
  (parent 332)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 624)
  (parent 332)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 625)
  (parent 332)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 626)
  (parent 333)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB e l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 627)
  (parent 333)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 628)
  (parent 333)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 629)
  (parent 333)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 630)
  (parent 333)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 631)
  (parent 333)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 632)
  (parent 334)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 633)
  (parent 334)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 634)
  (parent 334)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 635)
  (parent 334)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 636)
  (parent 334)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 637)
  (parent 334)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 638)
  (parent 335)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 639)
  (parent 335)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 640)
  (parent 335)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 641)
  (parent 335)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 642)
  (parent 335)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 643)
  (parent 335)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 644)
  (parent 335)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 645)
  (parent 336)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-1)))
      (send (cat (gen) (mul eA (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 646)
  (parent 336)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-2)))
      (send (cat (gen) (mul eA (rec eB) e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 647)
  (parent 336)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 648)
  (parent 336)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 649)
  (parent 336)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 650)
  (parent 336)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep e-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul e (rec eB) e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul e (rec eB) e-0) (one)) (e eB) (e-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e-0) (w (mul (rec eB) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0)))))
  (label 651)
  (parent 337)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 e eB e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e e) (lp l-1) (ep (mul e-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul e (rec eB) e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-1)))
        key)))
  (neq ((mul e (rec eB) e-0 l-1) (one)) (e eB) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eB e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 652)
  (parent 337)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul e-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul e (rec eB) e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        key)))
  (neq ((mul e (rec eB) e-0 l-2) (one)) (e eB) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 653)
  (parent 337)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB))))))
  (label 654)
  (parent 337)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 655)
  (parent 337)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 656)
  (parent 338)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 eA e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e e-0) (lp l-1) (ep (mul eA e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA e (rec e-0) l-1) (one)) (eA e-0) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 657)
  (parent 338)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))
        key)))
  (neq ((mul eA e (rec e-0) l-2) (one)) (eA e-0) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 658)
  (parent 338)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 659)
  (parent 338)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 660)
  (parent 338)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 661)
  (parent 338)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA))) key)))
  (neq ((mul e eA (rec eB)) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 662)
  (parent 339)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (e eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-1))) key)))
  (neq ((mul e eA (rec eB) l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 663)
  (parent 339)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (e eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2))) key)))
  (neq ((mul e eA (rec eB) l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 664)
  (parent 339)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 665)
  (parent 339)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 666)
  (parent 339)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul eA (rec eB) e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 667)
  (parent 340)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 668)
  (parent 340)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (lp expt) (l-2 rndx) (lp-0 expt)
    (eA eB e e-0 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 669)
  (parent 340)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w e-0))
    (mul eB (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 670)
  (parent 340)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 671)
  (parent 340)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 672)
  (parent 340)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 673)
  (parent 340)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 674)
  (parent 341)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 675)
  (parent 341)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 676)
  (parent 341)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 677)
  (parent 341)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 678)
  (parent 341)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 679)
  (parent 341)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 680)
  (parent 341)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 681)
  (parent 342)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 682)
  (parent 342)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 683)
  (parent 342)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 684)
  (parent 342)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 685)
  (parent 342)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 686)
  (parent 342)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 687)
  (parent 342)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 688)
  (parent 343)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 689)
  (parent 343)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 690)
  (parent 343)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 691)
  (parent 343)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-2))
    (mul eB (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 692)
  (parent 343)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 693)
  (parent 343)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-2))
    (mul (rec eA) eB w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 694)
  (parent 343)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 695)
  (parent 343)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 696)
  (parent 344)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l l-0)))
      (send (cat (gen) (mul eA (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 697)
  (parent 344)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 698)
  (parent 344)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 699)
  (parent 344)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 700)
  (parent 344)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 701)
  (parent 344)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 702)
  (parent 345)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (e eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0))) key)))
  (neq ((mul e (rec eB) l l-0) (one)) (e eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 703)
  (parent 345)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 704)
  (parent 345)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 705)
  (parent 345)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 706)
  (parent 345)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 707)
  (parent 346)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eA l (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA l (rec e) l-0) (one)) (eA e) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 708)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 709)
  (parent 346)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 710)
  (parent 346)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 711)
  (parent 346)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 712)
  (parent 346)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 713)
  (parent 347)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 714)
  (parent 347)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 715)
  (parent 347)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 716)
  (parent 347)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 717)
  (parent 347)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 718)
  (parent 347)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 719)
  (parent 347)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 720)
  (parent 348)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-0))) key)))
  (neq ((mul l eA (rec eB) l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 721)
  (parent 348)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 722)
  (parent 348)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 723)
  (parent 348)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 724)
  (parent 348)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 725)
  (parent 349)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 726)
  (parent 349)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 727)
  (parent 349)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 728)
  (parent 349)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 729)
  (parent 349)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 730)
  (parent 349)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 731)
  (parent 349)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 732)
  (parent 350)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 733)
  (parent 350)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 734)
  (parent 350)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 735)
  (parent 350)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 736)
  (parent 350)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 737)
  (parent 350)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 738)
  (parent 350)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 739)
  (parent 350)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 740)
  (parent 351)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 741)
  (parent 351)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 742)
  (parent 351)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 743)
  (parent 351)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 744)
  (parent 351)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 745)
  (parent 351)
  (unrealized (0 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 746)
  (parent 352)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0))) key)))
  (neq ((mul e (rec eB) l-0 l-0) (one)) (e eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 747)
  (parent 352)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 748)
  (parent 352)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 749)
  (parent 352)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 750)
  (parent 352)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 751)
  (parent 353)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) (eA e) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 752)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 753)
  (parent 353)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 754)
  (parent 353)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 755)
  (parent 353)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 756)
  (parent 353)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 757)
  (parent 354)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 758)
  (parent 354)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 759)
  (parent 354)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 760)
  (parent 354)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 761)
  (parent 354)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 762)
  (parent 354)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 763)
  (parent 354)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 764)
  (parent 355)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 765)
  (parent 355)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 766)
  (parent 355)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 767)
  (parent 355)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 768)
  (parent 355)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 769)
  (parent 355)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 770)
  (parent 355)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 771)
  (parent 356)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 772)
  (parent 356)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 773)
  (parent 356)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 774)
  (parent 356)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 775)
  (parent 356)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 776)
  (parent 357)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 777)
  (parent 357)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 778)
  (parent 357)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 779)
  (parent 357)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 780)
  (parent 357)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 781)
  (parent 357)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 782)
  (parent 357)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 783)
  (parent 357)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1)))))
  (label 784)
  (parent 358)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 785)
  (parent 358)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 786)
  (parent 358)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-2)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 787)
  (parent 358)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 788)
  (parent 358)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 789)
  (parent 358)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 790)
  (parent 358)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul (rec eB) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1)))))
  (label 791)
  (parent 359)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 792)
  (parent 359)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) key)))
  (neq ((mul e (rec eB) l-1 l-1) (one)) (e eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 793)
  (parent 359)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e (rec eB) l-1 l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 794)
  (parent 359)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 795)
  (parent 359)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 796)
  (parent 359)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA l-1 (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 797)
  (parent 360)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 798)
  (parent 360)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) (eA e) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 799)
  (parent 360)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 800)
  (parent 360)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 801)
  (parent 360)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 802)
  (parent 360)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 803)
  (parent 360)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul eA (rec eB) l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 804)
  (parent 361)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer self-1 name) (l l-0 rndx)
    (lp expt) (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 805)
  (parent 361)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 806)
  (parent 361)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 807)
  (parent 361)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-2) w e))
    (mul eB (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 808)
  (parent 361)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 809)
  (parent 361)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 810)
  (parent 361)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-2) w))
    (mul (rec eA) eB (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 811)
  (parent 361)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 812)
  (parent 362)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l l-1)))
        key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 813)
  (parent 362)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 814)
  (parent 362)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 815)
  (parent 362)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 816)
  (parent 362)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 817)
  (parent 362)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 818)
  (parent 362)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 819)
  (parent 362)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 820)
  (parent 363)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 821)
  (parent 363)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 822)
  (parent 363)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 823)
  (parent 363)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 824)
  (parent 363)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 825)
  (parent 363)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 826)
  (parent 363)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 827)
  (parent 363)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB))))))
  (label 828)
  (parent 364)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 829)
  (parent 364)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 830)
  (parent 364)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 831)
  (parent 364)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 832)
  (parent 364)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 833)
  (parent 364)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul eA (rec eB) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 834)
  (parent 365)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 835)
  (parent 365)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 836)
  (parent 365)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 837)
  (parent 365)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 838)
  (parent 365)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w l-2))
    (mul eB (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 839)
  (parent 365)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 840)
  (parent 365)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 841)
  (parent 365)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (added-absence l-2 (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 842)
  (parent 365)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 843)
  (parent 366)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 844)
  (parent 366)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 845)
  (parent 366)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 846)
  (parent 367)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 847)
  (parent 367)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eB) (send eB)))
  (label 848)
  (parent 367)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec e) l-1) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 849)
  (parent 368)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 850)
  (parent 368)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 851)
  (parent 368)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 852)
  (parent 369)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 853)
  (parent 369)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 854)
  (parent 369)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 855)
  (parent 369)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 856)
  (parent 370)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 857)
  (parent 370)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 858)
  (parent 370)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 859)
  (parent 370)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 860)
  (parent 371)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 861)
  (parent 371)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 862)
  (parent 371)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 863)
  (parent 371)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 864)
  (parent 372)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 865)
  (parent 372)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 866)
  (parent 372)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 867)
  (parent 372)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 868)
  (parent 372)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 869)
  (parent 373)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eB) e e l-1)))
      (send (cat (gen) (mul (rec eB) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 870)
  (parent 373)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 871)
  (parent 373)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 872)
  (parent 373)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 873)
  (parent 374)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 874)
  (parent 374)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 875)
  (parent 374)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 876)
  (parent 374)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 877)
  (parent 375)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e e (rec e-0) l-1) (one)) (e e-0) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 878)
  (parent 375)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 879)
  (parent 375)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 880)
  (parent 375)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul (rec eB) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 881)
  (parent 376)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 882)
  (parent 376)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 883)
  (parent 376)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 884)
  (parent 376)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 885)
  (parent 376)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 886)
  (parent 377)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 887)
  (parent 377)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 888)
  (parent 377)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 889)
  (parent 377)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 890)
  (parent 377)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 891)
  (parent 378)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 892)
  (parent 378)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 893)
  (parent 378)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 894)
  (parent 378)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 895)
  (parent 378)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 896)
  (parent 379)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 897)
  (parent 379)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 898)
  (parent 379)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 899)
  (parent 379)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 900)
  (parent 379)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 901)
  (parent 379)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 902)
  (parent 380)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-2)))
      (send (cat (gen) (mul eA (rec eB) e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 903)
  (parent 380)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 904)
  (parent 380)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 905)
  (parent 380)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 906)
  (parent 380)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep e-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul e (rec eB) e-0) (one)) (e eB) (e-0 (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e-0) (w (mul (rec eB) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0)))))
  (label 907)
  (parent 381)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul e-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        key)))
  (neq ((mul e (rec eB) e-0 l-2) (one)) (e eB) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 908)
  (parent 381)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB))))))
  (label 909)
  (parent 381)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 910)
  (parent 381)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 911)
  (parent 382)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))
        key)))
  (neq ((mul eA e (rec e-0) l-2) (one)) (eA e-0) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 912)
  (parent 382)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 913)
  (parent 382)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 914)
  (parent 382)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 915)
  (parent 382)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA))) key)))
  (neq ((mul e eA (rec eB)) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 916)
  (parent 383)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (e eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2))) key)))
  (neq ((mul e eA (rec eB) l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 917)
  (parent 383)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 918)
  (parent 383)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 919)
  (parent 383)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul eA (rec eB) e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 920)
  (parent 384)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (lp expt) (l-2 rndx) (lp-0 expt)
    (eA eB e e-0 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 921)
  (parent 384)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w e-0))
    (mul eB (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 922)
  (parent 384)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 923)
  (parent 384)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 924)
  (parent 384)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 925)
  (parent 384)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 926)
  (parent 385)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 927)
  (parent 385)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 928)
  (parent 385)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 929)
  (parent 385)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 930)
  (parent 385)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 931)
  (parent 385)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 932)
  (parent 386)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 933)
  (parent 386)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 934)
  (parent 386)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 935)
  (parent 386)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 936)
  (parent 386)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 937)
  (parent 386)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 938)
  (parent 387)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 939)
  (parent 387)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 940)
  (parent 387)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-2))
    (mul eB (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 941)
  (parent 387)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 942)
  (parent 387)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-2))
    (mul (rec eA) eB w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 943)
  (parent 387)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 944)
  (parent 387)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 945)
  (parent 388)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 946)
  (parent 388)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 947)
  (parent 388)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 948)
  (parent 388)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 949)
  (parent 388)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 950)
  (parent 389)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 951)
  (parent 389)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 952)
  (parent 389)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 953)
  (parent 389)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 954)
  (parent 390)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 955)
  (parent 390)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 956)
  (parent 390)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 957)
  (parent 390)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 958)
  (parent 390)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 959)
  (parent 391)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 960)
  (parent 391)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 961)
  (parent 391)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 962)
  (parent 391)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 963)
  (parent 391)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 964)
  (parent 391)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 965)
  (parent 392)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 966)
  (parent 392)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 967)
  (parent 392)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 968)
  (parent 392)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 969)
  (parent 393)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 970)
  (parent 393)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 971)
  (parent 393)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 972)
  (parent 393)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 973)
  (parent 393)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 974)
  (parent 393)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 975)
  (parent 394)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 976)
  (parent 394)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 977)
  (parent 394)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 978)
  (parent 394)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 979)
  (parent 394)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 980)
  (parent 394)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 981)
  (parent 394)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 982)
  (parent 395)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 983)
  (parent 395)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 984)
  (parent 395)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 985)
  (parent 395)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 986)
  (parent 395)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 987)
  (parent 396)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 988)
  (parent 396)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 989)
  (parent 396)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 990)
  (parent 396)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 991)
  (parent 397)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 992)
  (parent 397)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 993)
  (parent 397)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 994)
  (parent 397)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 995)
  (parent 397)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 996)
  (parent 398)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 997)
  (parent 398)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 998)
  (parent 398)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 999)
  (parent 398)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1000)
  (parent 398)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1001)
  (parent 398)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 1002)
  (parent 399)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1003)
  (parent 399)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 1004)
  (parent 399)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 1005)
  (parent 399)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 1006)
  (parent 399)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 1007)
  (parent 399)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 1008)
  (parent 400)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1009)
  (parent 400)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 1010)
  (parent 400)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 1011)
  (parent 400)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1012)
  (parent 401)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1013)
  (parent 401)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1014)
  (parent 401)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1015)
  (parent 401)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1016)
  (parent 401)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1017)
  (parent 401)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1018)
  (parent 401)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1)))))
  (label 1019)
  (parent 402)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1020)
  (parent 402)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-2)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1021)
  (parent 402)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 1022)
  (parent 402)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 1023)
  (parent 402)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 1024)
  (parent 402)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul (rec eB) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1)))))
  (label 1025)
  (parent 403)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) key)))
  (neq ((mul e (rec eB) l-1 l-1) (one)) (e eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1026)
  (parent 403)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e (rec eB) l-1 l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1027)
  (parent 403)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 1028)
  (parent 403)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 1029)
  (parent 403)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA l-1 (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 1030)
  (parent 404)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) (eA e) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1031)
  (parent 404)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1032)
  (parent 404)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 1033)
  (parent 404)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 1034)
  (parent 404)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 1035)
  (parent 404)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul eA (rec eB) l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1036)
  (parent 405)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1037)
  (parent 405)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1038)
  (parent 405)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-2) w e))
    (mul eB (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1039)
  (parent 405)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1040)
  (parent 405)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1041)
  (parent 405)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-2) w))
    (mul (rec eA) eB (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1042)
  (parent 405)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 1043)
  (parent 406)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1044)
  (parent 406)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1045)
  (parent 406)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 1046)
  (parent 406)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 1047)
  (parent 406)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 1048)
  (parent 406)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 1049)
  (parent 406)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 1050)
  (parent 407)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1051)
  (parent 407)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1052)
  (parent 407)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 1053)
  (parent 407)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 1054)
  (parent 407)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 1055)
  (parent 407)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 1056)
  (parent 407)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB))))))
  (label 1057)
  (parent 408)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1058)
  (parent 408)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1059)
  (parent 408)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 1060)
  (parent 408)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 1061)
  (parent 408)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul eA (rec eB) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1062)
  (parent 409)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1063)
  (parent 409)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1064)
  (parent 409)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1065)
  (parent 409)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w l-2))
    (mul eB (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1066)
  (parent 409)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1067)
  (parent 409)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1068)
  (parent 409)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (added-absence l-2 (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1069)
  (parent 409)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eB eA)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1070)
  (parent 411)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1071)
  (parent 412)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1072)
  (parent 414)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1073)
  (parent 417)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1074)
  (parent 418)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1075)
  (parent 420)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1076)
  (parent 421)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1077)
  (parent 422)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l))) (send (cat (gen) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 1078)
  (parent 424)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1079)
  (parent 425)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1080)
  (parent 429)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1081)
  (parent 430)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1082)
  (parent 431)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1083)
  (parent 434)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1084)
  (parent 435)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1085)
  (parent 438)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener e) (mul w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1086)
  (parent 439)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1087)
  (parent 442)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1088)
  (parent 443)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (e rndx)
    (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1089)
  (parent 446)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1090)
  (parent 447)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1091)
  (parent 448)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (e eB) ((mul (rec e) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1092)
  (parent 451)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul eB w l)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l e e))) key)))
  (neq ((mul w l e e) (one)) (e eB) ((mul eB w l e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l)))
      (send (cat (exp (gen) e) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w e e)))
      (send (cat (exp (gen) l) (mul w e e)))) ((recv e) (send e)))
  (label 1093)
  (parent 452)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1094)
  (parent 457)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv e) (send e)))
  (label 1095)
  (parent 458)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1096)
  (parent 459)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1097)
  (parent 462)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA e rndx)
    (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1098)
  (parent 463)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1099)
  (parent 464)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1100)
  (parent 467)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1101)
  (parent 468)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1102)
  (parent 471)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1103)
  (parent 472)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1104)
  (parent 473)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eB rndx) (w expt) (l-0 rndx)
    (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1105)
  (parent 476)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA rndx) (w expt) (l-0 rndx)
    (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1106)
  (parent 477)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0 eA))) key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1107)
  (parent 480)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1108)
  (parent 481)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (7 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0))) key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1109)
  (parent 482)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1110)
  (parent 483)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1111)
  (parent 486)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1112)
  (parent 487)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB rndx)
    (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) eB w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 e)))
        key)))
  (neq ((mul w l-0 e) (one)) (eA eB) ((mul (rec eA) eB w l-0 e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w e)))
      (send (cat (exp (gen) l-0) (mul w e)))) ((recv e) (send e)))
  (label 1113)
  (parent 488)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1114)
  (parent 493)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1115)
  (parent 494)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (7 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 e))) key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1116)
  (parent 495)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1117)
  (parent 496)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB (rec l) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1118)
  (parent 499)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1119)
  (parent 500)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1120)
  (parent 504)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1121)
  (parent 508)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1122)
  (parent 510)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB l-0 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eB l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1123)
  (parent 513)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA l-0 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1124)
  (parent 514)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e (mul (rec eA) eB (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1125)
  (parent 516)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w)))
      (send (cat (exp (gen) l) (mul eB w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 1126)
  (parent 519)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1127)
  (parent 520)
  (unrealized (0 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1128)
  (parent 524)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1129)
  (parent 525)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1130)
  (parent 527)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1131)
  (parent 531)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1132)
  (parent 532)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1133)
  (parent 533)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1134)
  (parent 533)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1135)
  (parent 537)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) eB))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 eB) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e l-0))) key)))
  (neq ((mul e l-0) (one)) (e eB) ((mul eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) eB)) (send (cat (exp (gen) l-0) eB)))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1136)
  (parent 538)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1137)
  (parent 538)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1138)
  (parent 542)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e l-0) (one)) (eA e) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1139)
  (parent 543)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1140)
  (parent 543)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1141)
  (parent 544)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1142)
  (parent 548)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1143)
  (parent 549)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA eB) ((mul (rec eA) eB e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1144)
  (parent 550)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (7 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1)))
  (label 1145)
  (parent 550)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1146)
  (parent 551)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1147)
  (parent 555)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1148)
  (parent 556)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((mul l l-0) (one)) (eA eB) ((mul (rec eA) eB l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1149)
  (parent 557)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0))) key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1150)
  (parent 557)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1151)
  (parent 562)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1152)
  (parent 563)
  (unrealized (0 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1153)
  (parent 568)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1154)
  (parent 569)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-1))) key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1155)
  (parent 570)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (7 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1156)
  (parent 570)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1157)
  (parent 571)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1158)
  (parent 574)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1159)
  (parent 575)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1160)
  (parent 578)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1161)
  (parent 582)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1162)
  (parent 583)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1163)
  (parent 586)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1164)
  (parent 587)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1165)
  (parent 588)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1166)
  (parent 591)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1167)
  (parent 592)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1168)
  (parent 596)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1169)
  (parent 597)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1170)
  (parent 598)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1171)
  (parent 602)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1172)
  (parent 603)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1173)
  (parent 604)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1174)
  (parent 608)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1175)
  (parent 609)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1176)
  (parent 613)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1177)
  (parent 614)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1178)
  (parent 618)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1179)
  (parent 619)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1180)
  (parent 623)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1181)
  (parent 624)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1182)
  (parent 625)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1183)
  (parent 629)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1184)
  (parent 630)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1185)
  (parent 635)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1186)
  (parent 636)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1187)
  (parent 637)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1188)
  (parent 642)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1189)
  (parent 643)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1190)
  (parent 644)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1191)
  (parent 648)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1192)
  (parent 649)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1193)
  (parent 650)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1194)
  (parent 654)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1195)
  (parent 655)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1196)
  (parent 659)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1197)
  (parent 660)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1198)
  (parent 661)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1199)
  (parent 665)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1200)
  (parent 666)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1201)
  (parent 670)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1202)
  (parent 671)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1203)
  (parent 672)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1204)
  (parent 673)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1205)
  (parent 677)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1206)
  (parent 678)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1207)
  (parent 679)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1208)
  (parent 684)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1209)
  (parent 685)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1210)
  (parent 686)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 1) (1 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 1211)
  (parent 687)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1212)
  (parent 692)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1213)
  (parent 693)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1214)
  (parent 694)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1215)
  (parent 695)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1216)
  (parent 699)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1217)
  (parent 700)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1218)
  (parent 705)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1219)
  (parent 710)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1220)
  (parent 712)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1221)
  (parent 716)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1222)
  (parent 717)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1223)
  (parent 719)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1224)
  (parent 723)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1225)
  (parent 724)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1226)
  (parent 728)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1227)
  (parent 729)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1228)
  (parent 731)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1229)
  (parent 736)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1230)
  (parent 737)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1231)
  (parent 739)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1232)
  (parent 743)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1233)
  (parent 744)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1234)
  (parent 745)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1235)
  (parent 745)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1236)
  (parent 749)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) eB))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 eB) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-0))) key)))
  (neq ((mul e l-0) (one)) (e eB) ((mul eB l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) eB)) (send (cat (exp (gen) l-0) eB)))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1237)
  (parent 750)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1238)
  (parent 750)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1239)
  (parent 754)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e l-0) (one)) (eA e) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1240)
  (parent 755)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1241)
  (parent 755)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1242)
  (parent 756)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1243)
  (parent 760)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1244)
  (parent 761)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA eB) ((mul (rec eA) eB e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1245)
  (parent 762)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1)))
  (label 1246)
  (parent 762)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1247)
  (parent 763)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1248)
  (parent 767)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1249)
  (parent 768)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((mul l l-0) (one)) (eA eB) ((mul (rec eA) eB l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1250)
  (parent 769)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1251)
  (parent 769)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1252)
  (parent 774)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1253)
  (parent 775)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1254)
  (parent 780)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1255)
  (parent 781)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1256)
  (parent 782)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0))
    ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1257)
  (parent 782)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1258)
  (parent 783)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1259)
  (parent 788)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1260)
  (parent 789)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1261)
  (parent 790)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1262)
  (parent 790)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1263)
  (parent 795)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) eB))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eB) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (e eB) ((mul eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) eB)) (send (cat (exp (gen) l-1) eB)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1264)
  (parent 796)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1265)
  (parent 796)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1266)
  (parent 801)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA e) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1267)
  (parent 802)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1268)
  (parent 802)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1269)
  (parent 803)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1270)
  (parent 808)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1271)
  (parent 809)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-2))) key)))
  (neq ((mul e l-2) (one)) (eA eB) ((mul (rec eA) eB e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1272)
  (parent 810)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2)))
  (label 1273)
  (parent 810)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1274)
  (parent 811)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1275)
  (parent 816)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1276)
  (parent 817)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1277)
  (parent 818)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1278)
  (parent 818)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1279)
  (parent 824)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1280)
  (parent 825)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1281)
  (parent 826)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1282)
  (parent 826)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0))
    ((6 1) (1 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 1283)
  (parent 827)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1284)
  (parent 832)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1285)
  (parent 833)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1286)
  (parent 839)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1287)
  (parent 840)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((mul l-1 l-2) (one)) (eA eB) ((mul (rec eA) eB l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1288)
  (parent 841)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1289)
  (parent 841)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1290)
  (parent 842)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1291)
  (parent 844)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1292)
  (parent 845)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1293)
  (parent 847)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1294)
  (parent 850)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1295)
  (parent 851)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1296)
  (parent 853)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1297)
  (parent 854)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1298)
  (parent 855)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1299)
  (parent 857)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1300)
  (parent 858)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1301)
  (parent 861)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1302)
  (parent 862)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1303)
  (parent 866)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1304)
  (parent 867)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1305)
  (parent 868)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1306)
  (parent 871)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1307)
  (parent 872)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1308)
  (parent 875)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1309)
  (parent 876)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1310)
  (parent 879)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1311)
  (parent 880)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1312)
  (parent 883)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1313)
  (parent 884)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1314)
  (parent 885)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1315)
  (parent 888)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1316)
  (parent 889)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1317)
  (parent 893)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1318)
  (parent 894)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1319)
  (parent 899)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1320)
  (parent 900)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1321)
  (parent 901)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1322)
  (parent 904)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1323)
  (parent 905)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1324)
  (parent 906)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1325)
  (parent 909)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1326)
  (parent 910)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1327)
  (parent 913)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1328)
  (parent 914)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1329)
  (parent 915)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1330)
  (parent 918)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1331)
  (parent 919)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1332)
  (parent 922)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1333)
  (parent 923)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1334)
  (parent 924)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1335)
  (parent 925)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1336)
  (parent 928)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1337)
  (parent 929)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1338)
  (parent 930)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1339)
  (parent 934)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1340)
  (parent 935)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1341)
  (parent 936)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1342)
  (parent 941)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1343)
  (parent 942)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1344)
  (parent 943)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1345)
  (parent 944)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1346)
  (parent 947)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1347)
  (parent 948)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1348)
  (parent 952)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1349)
  (parent 956)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1350)
  (parent 958)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1351)
  (parent 961)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1352)
  (parent 962)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1353)
  (parent 964)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1354)
  (parent 967)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1355)
  (parent 968)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1356)
  (parent 971)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1357)
  (parent 972)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1358)
  (parent 978)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1359)
  (parent 979)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1360)
  (parent 981)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1361)
  (parent 984)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1362)
  (parent 985)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1363)
  (parent 989)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1364)
  (parent 993)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1365)
  (parent 995)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1366)
  (parent 998)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1367)
  (parent 999)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1368)
  (parent 1001)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1369)
  (parent 1004)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1370)
  (parent 1005)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1371)
  (parent 1010)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1372)
  (parent 1011)
  (unrealized (0 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1373)
  (parent 1015)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1374)
  (parent 1016)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1375)
  (parent 1018)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1376)
  (parent 1022)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1377)
  (parent 1023)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1378)
  (parent 1024)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1379)
  (parent 1024)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1380)
  (parent 1028)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) eB))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eB) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (e eB) ((mul eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) eB)) (send (cat (exp (gen) l-1) eB)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1381)
  (parent 1029)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1382)
  (parent 1029)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1383)
  (parent 1033)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA e) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1384)
  (parent 1034)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1385)
  (parent 1034)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1386)
  (parent 1035)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1387)
  (parent 1039)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1388)
  (parent 1040)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-2))) key)))
  (neq ((mul e l-2) (one)) (eA eB) ((mul (rec eA) eB e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1389)
  (parent 1041)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2)))
  (label 1390)
  (parent 1041)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1391)
  (parent 1042)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1392)
  (parent 1046)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1393)
  (parent 1047)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1394)
  (parent 1048)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1395)
  (parent 1048)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1396)
  (parent 1053)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1397)
  (parent 1054)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1398)
  (parent 1055)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1399)
  (parent 1055)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1400)
  (parent 1060)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1401)
  (parent 1061)
  (unrealized (0 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1402)
  (parent 1066)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1403)
  (parent 1067)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((mul l-1 l-2) (one)) (eA eB) ((mul (rec eA) eB l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1404)
  (parent 1068)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1405)
  (parent 1068)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1406)
  (parent 1069)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1407)
  (parent 1082)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1408)
  (parent 1096)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1409)
  (parent 1117)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1410)
  (parent 1130)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1411)
  (parent 1134)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1412)
  (parent 1137)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1413)
  (parent 1140)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1414)
  (parent 1145)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0))) key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1415)
  (parent 1150)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1416)
  (parent 1156)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1417)
  (parent 1157)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1418)
  (parent 1170)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1419)
  (parent 1173)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1420)
  (parent 1187)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1421)
  (parent 1190)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1422)
  (parent 1211)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1423)
  (parent 1215)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1424)
  (parent 1228)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1425)
  (parent 1231)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1426)
  (parent 1235)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1427)
  (parent 1238)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1428)
  (parent 1241)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1429)
  (parent 1246)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1430)
  (parent 1251)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1431)
  (parent 1257)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1432)
  (parent 1258)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1433)
  (parent 1262)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1434)
  (parent 1265)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1435)
  (parent 1268)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1436)
  (parent 1273)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1437)
  (parent 1278)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1438)
  (parent 1282)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1439)
  (parent 1283)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1440)
  (parent 1289)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1441)
  (parent 1290)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1442)
  (parent 1305)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1443)
  (parent 1321)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1444)
  (parent 1345)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1445)
  (parent 1360)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1446)
  (parent 1375)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1447)
  (parent 1379)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1448)
  (parent 1382)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1449)
  (parent 1385)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1450)
  (parent 1390)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1451)
  (parent 1395)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1452)
  (parent 1399)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1453)
  (parent 1405)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1454)
  (parent 1406)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1455)
  (parent 1407)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1456)
  (parent 1407)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1457)
  (parent 1407)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1458)
  (parent 1408)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1459)
  (parent 1408)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1460)
  (parent 1408)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1461)
  (parent 1408)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1462)
  (parent 1409)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1463)
  (parent 1409)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1464)
  (parent 1409)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1465)
  (parent 1409)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1466)
  (parent 1409)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1467)
  (parent 1410)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1468)
  (parent 1410)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w))
    (mul eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1469)
  (parent 1410)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1470)
  (parent 1410)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0)))
        key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1471)
  (parent 1410)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec eB))))
      (send (cat (gen) (mul l-0 eA (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1472)
  (parent 1411)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec eB) l-1)))
      (send (cat (gen) (mul l-0 eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1473)
  (parent 1411)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1474)
  (parent 1411)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1475)
  (parent 1411)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1476)
  (parent 1412)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0 (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e l-0 (rec eB) l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1477)
  (parent 1412)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1478)
  (parent 1412)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener eB) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1479)
  (parent 1412)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1480)
  (parent 1413)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1481)
  (parent 1413)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0 eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1482)
  (parent 1413)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1483)
  (parent 1413)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1484)
  (parent 1414)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA l-2))) key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1485)
  (parent 1414)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1486)
  (parent 1414)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1487)
  (parent 1414)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1488)
  (parent 1414)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul l eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA))) key)))
  (neq ((mul l eA (rec eB)) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1489)
  (parent 1415)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul l eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA l-1)))
        key)))
  (neq ((mul l eA (rec eB) l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1490)
  (parent 1415)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1491)
  (parent 1415)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1492)
  (parent 1415)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1493)
  (parent 1415)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1494)
  (parent 1416)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1495)
  (parent 1416)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2))) key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1496)
  (parent 1416)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1497)
  (parent 1416)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1498)
  (parent 1416)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1499)
  (parent 1416)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1500)
  (parent 1417)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1))) key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1501)
  (parent 1417)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2))) key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1502)
  (parent 1417)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1503)
  (parent 1417)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1504)
  (parent 1417)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1505)
  (parent 1417)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1506)
  (parent 1418)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1507)
  (parent 1418)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1508)
  (parent 1418)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1509)
  (parent 1419)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1510)
  (parent 1419)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1511)
  (parent 1419)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1512)
  (parent 1419)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1513)
  (parent 1420)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1514)
  (parent 1420)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1515)
  (parent 1420)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1516)
  (parent 1420)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1517)
  (parent 1421)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1518)
  (parent 1421)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1519)
  (parent 1421)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1520)
  (parent 1421)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1521)
  (parent 1421)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1522)
  (parent 1422)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1523)
  (parent 1422)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1524)
  (parent 1422)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1525)
  (parent 1422)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1526)
  (parent 1422)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1527)
  (parent 1423)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1528)
  (parent 1423)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1529)
  (parent 1423)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1530)
  (parent 1423)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1531)
  (parent 1423)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1532)
  (parent 1423)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1533)
  (parent 1424)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1534)
  (parent 1424)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w))
    (mul eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1535)
  (parent 1424)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1536)
  (parent 1424)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1537)
  (parent 1424)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1538)
  (parent 1425)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1539)
  (parent 1425)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1540)
  (parent 1425)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1541)
  (parent 1425)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1542)
  (parent 1425)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1543)
  (parent 1425)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec eB))))
      (send (cat (gen) (mul l-0 eA (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1544)
  (parent 1426)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec eB) l-1)))
      (send (cat (gen) (mul l-0 eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1545)
  (parent 1426)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1546)
  (parent 1426)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1547)
  (parent 1426)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1548)
  (parent 1427)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0 (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e l-0 (rec eB) l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1549)
  (parent 1427)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1550)
  (parent 1427)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (3 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener eB) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1551)
  (parent 1427)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1552)
  (parent 1428)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1553)
  (parent 1428)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1554)
  (parent 1428)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1555)
  (parent 1428)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1556)
  (parent 1429)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1557)
  (parent 1429)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1558)
  (parent 1429)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1559)
  (parent 1429)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1560)
  (parent 1429)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul l eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA))) key)))
  (neq ((mul l eA (rec eB)) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1561)
  (parent 1430)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul l eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-1))) key)))
  (neq ((mul l eA (rec eB) l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1562)
  (parent 1430)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1563)
  (parent 1430)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1564)
  (parent 1430)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1565)
  (parent 1430)
  (unrealized (0 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1566)
  (parent 1431)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1567)
  (parent 1431)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1568)
  (parent 1431)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1569)
  (parent 1431)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1570)
  (parent 1431)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1571)
  (parent 1431)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1572)
  (parent 1432)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1573)
  (parent 1432)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1574)
  (parent 1432)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1575)
  (parent 1432)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1576)
  (parent 1432)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1577)
  (parent 1432)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec eB))))
      (send (cat (gen) (mul l-1 eA (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1578)
  (parent 1433)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec eB) l-1)))
      (send (cat (gen) (mul l-0 eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1579)
  (parent 1433)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec eB) l-2)))
      (send (cat (gen) (mul l-1 eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1580)
  (parent 1433)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1581)
  (parent 1433)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1582)
  (parent 1433)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1583)
  (parent 1434)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e l-0 (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e l-0 (rec eB) l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1584)
  (parent 1434)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e l-1 eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1 (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e l-1 (rec eB) l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1585)
  (parent 1434)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1586)
  (parent 1434)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1587)
  (parent 1434)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1588)
  (parent 1435)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul l-0 eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1589)
  (parent 1435)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1590)
  (parent 1435)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1591)
  (parent 1435)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1592)
  (parent 1435)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-2 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA))) key)))
  (neq ((mul l-2 eA (rec eB)) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1593)
  (parent 1436)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1594)
  (parent 1436)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-2 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))
        key)))
  (neq ((mul l-2 eA (rec eB) l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1595)
  (parent 1436)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1596)
  (parent 1436)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1597)
  (parent 1436)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1598)
  (parent 1436)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1599)
  (parent 1437)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul l eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eA l-1)))
        key)))
  (neq ((mul l eA (rec eB) l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1600)
  (parent 1437)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1601)
  (parent 1437)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1602)
  (parent 1437)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1603)
  (parent 1437)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1604)
  (parent 1437)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1605)
  (parent 1438)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1606)
  (parent 1438)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1607)
  (parent 1438)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1608)
  (parent 1438)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1609)
  (parent 1438)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1610)
  (parent 1438)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1611)
  (parent 1439)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1612)
  (parent 1439)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1613)
  (parent 1439)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1614)
  (parent 1439)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1615)
  (parent 1439)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1616)
  (parent 1439)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1617)
  (parent 1440)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1618)
  (parent 1440)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1619)
  (parent 1440)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1620)
  (parent 1440)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1621)
  (parent 1440)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1622)
  (parent 1440)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1623)
  (parent 1440)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul eA (rec eB) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1624)
  (parent 1441)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1625)
  (parent 1441)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1626)
  (parent 1441)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1627)
  (parent 1441)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1628)
  (parent 1441)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1629)
  (parent 1441)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1630)
  (parent 1441)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1631)
  (parent 1442)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1632)
  (parent 1442)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1633)
  (parent 1442)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1634)
  (parent 1443)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1635)
  (parent 1443)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1636)
  (parent 1443)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1637)
  (parent 1443)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1638)
  (parent 1444)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1639)
  (parent 1444)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1640)
  (parent 1444)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1641)
  (parent 1444)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1642)
  (parent 1444)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1643)
  (parent 1445)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1644)
  (parent 1445)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1645)
  (parent 1445)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1646)
  (parent 1445)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1647)
  (parent 1445)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1648)
  (parent 1446)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1649)
  (parent 1446)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1650)
  (parent 1446)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1651)
  (parent 1446)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1652)
  (parent 1446)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec eB))))
      (send (cat (gen) (mul l-1 eA (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1653)
  (parent 1447)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec eB) l-2)))
      (send (cat (gen) (mul l-1 eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1654)
  (parent 1447)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1655)
  (parent 1447)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1656)
  (parent 1447)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1657)
  (parent 1448)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e l-1 eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1 (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e l-1 (rec eB) l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1658)
  (parent 1448)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1659)
  (parent 1448)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1660)
  (parent 1448)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1661)
  (parent 1449)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1662)
  (parent 1449)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1663)
  (parent 1449)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1664)
  (parent 1449)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-2 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA))) key)))
  (neq ((mul l-2 eA (rec eB)) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1665)
  (parent 1450)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-2 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))
        key)))
  (neq ((mul l-2 eA (rec eB) l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1666)
  (parent 1450)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1667)
  (parent 1450)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1668)
  (parent 1450)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1669)
  (parent 1450)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1670)
  (parent 1451)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1671)
  (parent 1451)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1672)
  (parent 1451)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1673)
  (parent 1451)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1674)
  (parent 1451)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1675)
  (parent 1452)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1676)
  (parent 1452)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1677)
  (parent 1452)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1678)
  (parent 1452)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1679)
  (parent 1452)
  (unrealized (0 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1680)
  (parent 1453)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1681)
  (parent 1453)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1682)
  (parent 1453)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1683)
  (parent 1453)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1684)
  (parent 1453)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1685)
  (parent 1453)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul eA (rec eB) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1686)
  (parent 1454)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1687)
  (parent 1454)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1688)
  (parent 1454)
  (unrealized (1 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1689)
  (parent 1454)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1690)
  (parent 1454)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1691)
  (parent 1454)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1692)
  (parent 1456)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1693)
  (parent 1457)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1694)
  (parent 1460)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1695)
  (parent 1461)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1696)
  (parent 1464)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1697)
  (parent 1465)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1698)
  (parent 1466)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1699)
  (parent 1469)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1700)
  (parent 1470)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1701)
  (parent 1474)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1702)
  (parent 1475)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1703)
  (parent 1476)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1704)
  (parent 1478)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0 eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1705)
  (parent 1482)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1706)
  (parent 1483)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1707)
  (parent 1486)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1708)
  (parent 1487)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (8 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1709)
  (parent 1488)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1710)
  (parent 1491)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1711)
  (parent 1492)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1712)
  (parent 1497)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1713)
  (parent 1498)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (8 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1714)
  (parent 1499)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1715)
  (parent 1503)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1716)
  (parent 1504)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1717)
  (parent 1505)
  (seen 1806)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1718)
  (parent 1507)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1719)
  (parent 1508)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1720)
  (parent 1511)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1721)
  (parent 1512)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1722)
  (parent 1515)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1723)
  (parent 1516)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1724)
  (parent 1520)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1725)
  (parent 1521)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1726)
  (parent 1524)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1727)
  (parent 1525)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1728)
  (parent 1526)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1729)
  (parent 1530)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1730)
  (parent 1531)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec l-2) e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1731)
  (parent 1532)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1732)
  (parent 1535)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1733)
  (parent 1536)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1734)
  (parent 1541)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1735)
  (parent 1542)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1736)
  (parent 1546)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1737)
  (parent 1547)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1738)
  (parent 1548)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1739)
  (parent 1550)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1740)
  (parent 1554)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1741)
  (parent 1555)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1742)
  (parent 1558)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1743)
  (parent 1559)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (8 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1744)
  (parent 1560)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1745)
  (parent 1563)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1746)
  (parent 1564)
  (unrealized (0 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1747)
  (parent 1569)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1748)
  (parent 1570)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (8 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1749)
  (parent 1571)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1750)
  (parent 1575)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1751)
  (parent 1576)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (8 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 1752)
  (parent 1577)
  (seen 1807)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1753)
  (parent 1581)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1754)
  (parent 1582)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1755)
  (parent 1583)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1756)
  (parent 1586)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1757)
  (parent 1591)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1758)
  (parent 1592)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1759)
  (parent 1596)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1760)
  (parent 1597)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1761)
  (parent 1598)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1762)
  (parent 1602)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1763)
  (parent 1603)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1764)
  (parent 1608)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1765)
  (parent 1609)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (8 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 1766)
  (parent 1610)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1767)
  (parent 1614)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1768)
  (parent 1615)
  (unrealized (0 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1769)
  (parent 1616)
  (seen 1808)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1770)
  (parent 1621)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1771)
  (parent 1622)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1772)
  (parent 1623)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1773)
  (parent 1628)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1774)
  (parent 1629)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1775)
  (parent 1630)
  (seen 1809)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1776)
  (parent 1632)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1777)
  (parent 1633)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1778)
  (parent 1636)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1779)
  (parent 1637)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1780)
  (parent 1640)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1781)
  (parent 1641)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec l-2) e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1782)
  (parent 1642)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1783)
  (parent 1645)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1784)
  (parent 1646)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1785)
  (parent 1650)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1786)
  (parent 1651)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1787)
  (parent 1655)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1788)
  (parent 1656)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1789)
  (parent 1657)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1790)
  (parent 1659)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1791)
  (parent 1663)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1792)
  (parent 1664)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1793)
  (parent 1667)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1794)
  (parent 1668)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1795)
  (parent 1669)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1796)
  (parent 1672)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1797)
  (parent 1673)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1798)
  (parent 1677)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1799)
  (parent 1678)
  (unrealized (0 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1800)
  (parent 1683)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1801)
  (parent 1684)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1802)
  (parent 1685)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1803)
  (parent 1689)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1804)
  (parent 1690)
  (unrealized (0 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1805)
  (parent 1691)
  (seen 1810)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1806)
  (parent 1714)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1807)
  (parent 1749)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1808)
  (parent 1766)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1809)
  (parent 1772)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1810)
  (parent 1802)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1811)
  (parent 1806)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2))) key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1812)
  (parent 1806)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1813)
  (parent 1806)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1814)
  (parent 1806)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1815)
  (parent 1807)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (4 0))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1816)
  (parent 1807)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1817)
  (parent 1807)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1818)
  (parent 1807)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1819)
  (parent 1808)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 0) (4 0))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1820)
  (parent 1808)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1821)
  (parent 1808)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1822)
  (parent 1808)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1823)
  (parent 1809)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (0 1)) ((9 0) (1 1))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 3 10 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1824)
  (parent 1809)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1825)
  (parent 1809)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1826)
  (parent 1809)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1827)
  (parent 1809)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1828)
  (parent 1810)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1829)
  (parent 1810)
  (unrealized (1 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1830)
  (parent 1810)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1831)
  (parent 1810)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1832)
  (parent 1813)
  (unrealized (0 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-0 (rec l-1) (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1833)
  (parent 1814)
  (unrealized (0 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1834)
  (parent 1817)
  (unrealized (0 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-0 (rec l-1) (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1835)
  (parent 1818)
  (unrealized (0 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1836)
  (parent 1821)
  (unrealized (0 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-0 (rec l-1) (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1837)
  (parent 1822)
  (unrealized (0 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1838)
  (parent 1826)
  (unrealized (0 3) (4 0) (10 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-1 (rec l-2) (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1839)
  (parent 1827)
  (unrealized (0 3) (4 0) (10 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1840)
  (parent 1830)
  (unrealized (0 3) (4 0) (10 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-1 (rec l-2) (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1841)
  (parent 1831)
  (unrealized (0 3) (4 0) (10 0))
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (lp ep expt) (self peer name) (key data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ep)) (send key))
    (uniq-gen e)
    (fn-of
      (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e ep))) key)))
    (neq (ep (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) lp))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))))
  (label 1842)
  (unrealized (0 0) (0 1) (0 3) (2 0))
  (preskeleton)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (precedes ((0 2) (2 0)) ((1 2) (0 3)) ((1 2) (2 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))))
  (label 1843)
  (parent 1842)
  (unrealized (0 0) (0 1) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (precedes ((0 2) (1 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (operation collapsed 1 0)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))))
  (label 1844)
  (parent 1843)
  (unrealized (0 0) (0 1) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
    (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))))
  (label 1845)
  (parent 1843)
  (unrealized (0 0) (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
    (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))))
  (label 1846)
  (parent 1844)
  (unrealized (0 0) (0 1) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 w expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul e1 e2 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e1 e2 (rec w))) w))
    (exp (gen) (mul e1 e2)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul e1 e2 (rec w))) w))
      (send (cat (exp (gen) (mul e1 e2 (rec w))) w))))
  (label 1847)
  (parent 1845)
  (unrealized (0 0) (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx) (w expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul e2 e2 (rec w))) w))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e2 e2 (rec w))) w))
    (exp (gen) (mul e2 e2)) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul e2 e2 (rec w))) w))
      (send (cat (exp (gen) (mul e2 e2 (rec w))) w))))
  (label 1848)
  (parent 1846)
  (unrealized (0 0) (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (l rndx) (lp expt) (l-0 rndx)
    (lp-0 expt) (e1 e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (gen) (mul e1 e2)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (precur (4 0))
  (operation nonce-test (contracted (e1-0 e1) (e2-0 e2) (w (mul e1 e2)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (gen) (mul e1 e2))) (send (cat (gen) (mul e1 e2)))))
  (label 1849)
  (parent 1847)
  (unrealized (0 0) (0 1) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (l rndx) (lp expt) (e2 l-0 rndx)
    (lp-0 expt) (e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l-0) (e e)
    (lp lp-0) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e2) (lp lp))
  (deflistener (hash (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e))))
  (deflistener (cat (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e))))
  (deflistener (cat (exp (gen) e) e2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp-0)) (e2 (exp (gen) l))
    (e2 (exp (gen) lp)))
  (fn-of
    (foo
      ((hash (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e2 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) lp-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e))))
      (send (hash (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e)))))
    ((recv (cat (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e))))
      (send (cat (exp (gen) (mul l-0 lp-0)) (exp (gen) (mul e2 e)))))
    ((recv (cat (exp (gen) e) e2)) (send (cat (exp (gen) e) e2))))
  (label 1850)
  (parent 1847)
  (unrealized (0 0) (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (l rndx) (lp expt) (e1 l-0 rndx)
    (lp-0 expt) (e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e))))
  (deflistener (cat (exp (gen) e) e1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e))) key)))
  (neq (e (one)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e1 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e)))))
    ((recv (cat (exp (gen) e) e1)) (send (cat (exp (gen) e) e1))))
  (label 1851)
  (parent 1847)
  (unrealized (0 0) (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer self-0 peer-0 name) (l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt) (l-1 rndx) (lp-1 expt) (e1 e2 e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) e) (mul e1 e2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e)
    (lp lp-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp-1)) (e2 (exp (gen) l-0))
    (e2 (exp (gen) lp-0)) (e1 (exp (gen) l)) (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e1 e2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) e) (mul e1 e2 (rec e))))
      (send (cat (exp (gen) e) (mul e1 e2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-1) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 1852)
  (parent 1847)
  (unrealized (0 0) (0 1) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer self-0 name) (l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt) (e1 e2 l-1 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) l-1) (mul e1 e2 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 0) (4 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of ("principal-of" (l-1 self-0))
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e1 e2 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) l-1) (mul e1 e2 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul e1 e2 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1853)
  (parent 1847)
  (unrealized (0 0) (0 1) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (gen) (mul e2 e2)))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test (contracted (e2-0 e2) (w (mul e2 e2))) (gen)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (gen) (mul e2 e2))) (send (cat (gen) (mul e2 e2)))))
  (label 1854)
  (parent 1848)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e)
    (lp lp) (ep e))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
  (deflistener (cat (exp (gen) e) e))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e e))) key)))
  (neq (e (one)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e e)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e e)))))
    ((recv (cat (exp (gen) e) e)) (send (cat (exp (gen) e) e))))
  (label 1855)
  (parent 1848)
  (unrealized (0 0) (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (l rndx) (lp expt) (l-0 rndx)
    (lp-0 expt) (e2 e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) e) (mul e2 e2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp-0)) (e2 (exp (gen) l))
    (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) e) (mul e2 e2 (rec e))))
      (send (cat (exp (gen) e) (mul e2 e2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1856)
  (parent 1848)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self name) (l rndx) (lp expt) (e2 l-0 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) l-0) (mul e2 e2 (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (3 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of ("principal-of" (l-0 self))
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e2 l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) l-0) (mul e2 e2 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e2 e2 (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1857)
  (parent 1848)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (gen) (mul e2 e2)))
  (deflistener e2)
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test (added-listener e2) (mul e2 e2) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (gen) (mul e2 e2))) (send (cat (gen) (mul e2 e2))))
    ((recv e2) (send e2)))
  (label 1858)
  (parent 1854)
  (unrealized (0 0) (0 1) (4 0))
  (comment "empty cohort"))

(comment "Nothing left to do")
